// bytran -|- spectral calculations for portable devices using the HITRAN database
//
// Developed by Dzianis Pliutau (Denis Plutov) and Kostya Roslyakov
// www.bytran.org | contact@bytran.org
// Copyright (c) 2016 Dzianis Pliutau
// Distributed under the MIT license
//
// The MIT License (MIT)
// Copyright (c) 2016 Dzianis Pliutau
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
// is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
// IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.



// QML component is used to specify single spectral line options to be used in calculations
import QtQuick 2.7
import QtQuick.Controls 2.0
import QtQuick.Layouts 1.0

Item {

    id: lineOptionsItem
    anchors.fill: parent

    // Rectangle to shade the background
    Rectangle {
        anchors.fill: parent
        border.width: 1
        border.color: 'black'
        color: Qt.rgba(0, 0, 0, 0.6)

        // This is introduced to disable items located on the background
        MouseArea {
            anchors.fill: parent
            onClicked: { }
        }
    }

    // Rectangle to draw the form border
    Rectangle {
        anchors.fill: mainLineOptionsColumnLayout
        anchors.margins: coreObj.defaultLabelHeight * (-0.85)
        color: "white"
        border.width: 1
        border.color: 'gray'
        radius: coreObj.defaultLabelHeight * 0.5
    }

    // Exit button
    CustomButton {
        id: exitButton
        anchors.top: lineOptionsItem.top
        anchors.right: lineOptionsItem.right
        anchors.margins: coreObj.defaultLabelHeight * (0.1)
        implicitWidth: 1.5 * exitButton.height
        text: qsTr("X")
        onClicked: lineOptionsItem.destroy()
    }

    CustomLabel {
        id: widthLabel
        text: "Single spectral line calculation setup options"
        visible: false
        enabled: false
    }

    // Column layout to hold all the interface items
    ColumnLayout {
        id: mainLineOptionsColumnLayout
        anchors.centerIn: parent        
        Layout.fillWidth: true
        Layout.fillHeight: true
        spacing: 5

        CustomLabel {
            id: commentLabel
            Layout.fillWidth: true            
            text: qsTr("Single spectral line calculation setup options")
        }

        CustomComboBox {
            id: lineShapeComboBox
            Layout.fillWidth: true
            implicitWidth: widthLabel.width
            model: [ "Doppler lineshape profile", "Lorentz lineshape profile", "Voigt lineshape / Standard precision", "Voigt lineshape / Higher precision"]
            currentIndex: lineOptDataObj.lineshapeProfile
            onCurrentIndexChanged: lineOptDataObj.lineshapeProfile = currentIndex
            font.pointSize: coreObj.defaultFontPointSize - 1
        }

        GridLayout {
            id: mainLineshapeGridLayout
            rows: 5
            columns: 2

            // Intensity threshold
            CustomLabel {
                id: intensityThresholdLabel
                anchors.right: intensityThresholdTextField.left
                anchors.rightMargin: coreObj.defaultLabelHeight * 0.3
                Layout.column: 0
                Layout.row: 0
                horizontalAlignment: Text.AlignRight
                text: qsTr("Intensity threshold:\n(cm * molec * 1E-25)")
            }
            CustomTextField {
                id: intensityThresholdTextField
                Layout.column: 1
                Layout.row: 0
                implicitWidth: widthLabel.width - wingContribAbsLabel.width                
                text: lineOptDataObj.intensThresh
                onTextChanged: lineOptDataObj.intensThresh = text
            }

            // Line wing contribution in halfwidths
            CustomLabel {
                id: wingContribSatHwLabel
                anchors.right: wingContribSatHwTextField.left
                anchors.rightMargin: coreObj.defaultLabelHeight * 0.3
                Layout.column: 0
                Layout.row: 1
                horizontalAlignment: Text.AlignRight
                text: qsTr("Line wing contribution:\n(saturated halfwidths)")
            }
            CustomTextField {
                id: wingContribSatHwTextField
                Layout.column: 1
                Layout.row: 1
                implicitWidth: widthLabel.width - wingContribAbsLabel.width
                text: lineOptDataObj.wingContribSatHw
                onTextChanged: lineOptDataObj.wingContribSatHw = text
            }

            // Line wing contribution absolute
            CustomLabel {
                id: wingContribAbsLabel
                anchors.right: wingContribAbsTextField.left
                anchors.rightMargin: coreObj.defaultLabelHeight * 0.3
                Layout.column: 0
                Layout.row: 2
                horizontalAlignment: Text.AlignRight
                text: qsTr("Line wing contribution:\n(absolute, in cm-1)")
            }
            // Changing the line wing contribution absolute
            CustomTextField {
                id: wingContribAbsTextField
                Layout.column: 1
                Layout.row: 2
                implicitWidth: widthLabel.width - wingContribAbsLabel.width
                text: lineOptDataObj.wingContribAbs
                onTextChanged: lineOptDataObj.wingContribAbs = text
            }

            ColumnLayout {
                id: broadeningColumnLayout
                anchors.left: mainLineshapeGridLayout.left
                anchors.right: wingContribAbsTextField.left
                anchors.rightMargin: coreObj.defaultLabelHeight * 0.3
                Layout.column: 0
                Layout.row: 3

                // Broadening type selection
                CustomLabel {
                    id: broadeningLabel
                    text: qsTr("Lineshape broadening")
                }

                CustomComboBox {
                    id: broadeningComboBox
                    Layout.fillWidth: true
                    model: [ "Air", "Self", "Air and Self"]
                    currentIndex: lineOptDataObj.lineBroadening
                    onCurrentIndexChanged: lineOptDataObj.lineBroadening = currentIndex
                }
            }

            RowLayout {
                id: lineshiftRowLayout
                anchors.left: broadeningColumnLayout.right
                anchors.right: mainLineshapeGridLayout.right
                anchors.leftMargin: coreObj.defaultLabelHeight * 0.3
                Layout.column: 1
                Layout.row: 3

                Rectangle {
                    id: separatorRect
                    anchors.left: lineshiftRowLayout
                    Layout.fillHeight: true
                    width: 1
                    color: '#C0C0C0'
                }

                ColumnLayout {
                    id: lineshiftColumnLayout
                    anchors.left: separatorRect.right
                    anchors.right: lineshiftRowLayout.right

                    // Broadening
                    RowLayout {
                        id: switchRowLayout
                        anchors.left: lineshiftColumnLayout.left
                        anchors.right: lineshiftColumnLayout.right

                        CustomLabel {
                            id: onRowLabel
                            anchors.right: lineShiftSwitch.left
                            text: qsTr("No")
                            color: 'gray'
                            MouseArea {
                                anchors.fill: parent
                                onClicked: lineShiftSwitch.checked = false
                            }
                        }
                        CustomSwitch {
                            id: lineShiftSwitch
                            anchors.centerIn: parent
                            padding: 0
                            checked: lineOptDataObj.useLineShift
                            onCheckedChanged: lineOptDataObj.useLineShift = checked                            
                        }
                        CustomLabel {
                            id: offRowLabel
                            anchors.left: lineShiftSwitch.right
                            text: qsTr("Yes");
                            color: 'red'
                            MouseArea {
                                anchors.fill: parent
                                onClicked: lineShiftSwitch.checked = true
                            }
                        }
                    }

                    // Comment label
                    RowLayout {
                        anchors.left: lineshiftColumnLayout.left
                        anchors.right: lineshiftColumnLayout.right
                        CustomLabel {
                            id: useLineShiftLabel
                            anchors.centerIn: parent
                            text: qsTr("Use line shift")
                            MouseArea {
                                anchors.fill: parent
                                onClicked: lineShiftSwitch.checked = !lineShiftSwitch.checked
                            }
                        }
                    }
                }
            }
        }
    }
}

