// bytran -|- spectral calculations for portable devices using the HITRAN database
//
// Developed by Dzianis Pliutau (Denis Plutov) and Kostya Roslyakov
// www.bytran.org | contact@bytran.org
// Copyright (c) 2016 Dzianis Pliutau
// Distributed under the MIT license
//
// The MIT License (MIT)
// Copyright (c) 2016 Dzianis Pliutau
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
// is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
// IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.



// Main QML window
import QtQuick 2.7
import QtQuick.Controls 2.0
import QtQuick.Layouts 1.0
import Plot 1.0

Item {

    // Custom components are not used here because changing the application font in the main.cpp affects this
    // form in terms of the default font. Maybe will change this later to have custom components here as well.

    id: mainWindowItem

    property alias menuButton: menuButton
    property alias calcButton: calcButton
    property alias exitButton: exitButton
    property alias moleculesButton: moleculesButton
    property alias spectrumPlot: spectrumPlot
    property alias plotAndMolViewRect: plotAndMolViewRect
    property alias totalMolsButton: totalMolsButton
    property alias menuButtonText: menuButtonText
    property alias totalsButtonRect: totalsButtonRect
    property alias plotPinchArea: plotPinchArea
    property alias showTotalOptionsRectangle: showTotalOptionsRectangle
    property alias showTotalOrRadianceButton: showTotalOrRadianceButton
    property alias heightLabel: heightLabel
    property alias okDialogTextLabel: okDialogTextLabel

    // Using the custom label here for iOS
    CustomSizingLabel {
        id: heightLabel
        text: qsTr("  Atmosphere model  ")
        visible: false
        enabled: false
    }

    // Custom label to assign the text to the ok dialog and also open the Ok dialog
    CustomLabel {
        id: okDialogTextLabel
        text: coreObj.okDialogText
        visible: false
        enabled: false
    }

    // Rectangle to contain the calculated plot
    Rectangle {
        id: plotAndMolViewRect
        anchors.fill: parent
        color: "white"
        border.width: 1
        border.color: "white"
        radius: 3

        // Plot to contain the plotted spectra (will also be used to manipulate (scaling, resizing etc.) the spectra through touch etc.)
        Plot {
            id: spectrumPlot
            anchors.fill: parent
            xMinPadding: menuButton.width * 0.87
            xMaxPadding: menuButton.width * 1.02
            yMinPadding: menuButton.height + 2 * menuButton.anchors.topMargin
            yMaxPadding: menuButton.height * 1.23

            yAxisTitle: specResultsDataObj.yAxisTitle
            xAxisTitle: specResultsDataObj.xAxisTitle
            minX: specResultsDataObj.minX
            maxX: specResultsDataObj.maxX
            minY: specResultsDataObj.minY
            maxY: specResultsDataObj.maxY            
            numMajorXTicks: 5
            numMinorXTicks: 5
            numMajorYTicks: 5
            numMinorYTicks: 5

            multiplier: specResultsDataObj.multiplier
            spectrum: specResultsDataObj.spectrum
            currSpecIndex: specResultsDataObj.currSpecIndex
            wnGrid: specResultsDataObj.wnGrid
            lineColor: specResultsDataObj.lineColor
            plotInverted: false
            lineThickness: 1            

            saveImage: shareResultsDataObj.saveImage;
            topString: shareResultsDataObj.topString;
            fileWritingComplete: shareResultsDataObj.fileWritingComplete;

            // Used to enable zoom of the plot and other functions
            PinchArea {
                id: plotPinchArea
                anchors.fill: parent
            }
        }

        Rectangle {
            id: totalsButtonRect
            anchors.bottom: plotAndMolViewRect.bottom
            anchors.bottomMargin: menuButton.height * 1.22
            anchors.right: plotAndMolViewRect.right
            height: totalMolsButton.height * 1.15
            color: 'white'
            width: molListView.width
        }

        ListView {
            id: molListView
            anchors.top: plotAndMolViewRect.top
            anchors.topMargin: (Qt.platform.os == 'ios') ? menuButton.height * 1.02 + 2 : menuButton.height * 1.02
            anchors.right: plotAndMolViewRect.right
            anchors.bottom: totalsButtonRect.top
            implicitWidth: menuButton.width
            model: molListViewReducedModel
            visible: !specResultsDataObj.showTotal
            enabled: true
            spacing: 0
            clip: true

            delegate:
            RowLayout {
                id: moleculeDelegate
                implicitWidth: menuButton.width

                RowLayout {
                    id: molColorRowLayout
                    // Rectangle to draw the color to be plotted
                    Rectangle {
                        width: menuButton.width
                        height: indexText.height
                        border.width: 1
                        border.color: model.color
                        color: model.color
                        radius: 1
                        CustomSmallerFontText {
                            id: indexText
                            horizontalAlignment: Text.AlignRight
                            text: " " + model.molName
                            font.bold: true                            
                            color: 'white'
                        }
                    }
                }
            }
        }

        // Menu buttons size has been minimized to enlarge the area available for the plot
        CustomButton {
            id: menuButton
            anchors.left: spectrumPlot.left
            anchors.top: spectrumPlot.top
            anchors.margins: menuButton.height * 0.04
            anchors.leftMargin: menuButton.width * 0.87
            implicitHeight: ((Qt.platform.os == 'ios') || (Qt.platform.os == 'winrt') || (Qt.platform.os == 'winphone')) ? menuButtonText.height * 1.8 : menuButtonText.height * 2
            implicitWidth: menuButtonText.width * 1.5
            CustomText {
                id: menuButtonText
                anchors.centerIn: parent                
                text: qsTr("Menu")
            }            
        }
        CustomButton {
            id: calcButton
            anchors.left: menuButton.right
            anchors.top: spectrumPlot.top
            anchors.margins: menuButton.anchors.topMargin
            anchors.leftMargin: menuButtonText.width * 0.5
            implicitHeight: menuButton.height
            implicitWidth: menuButton.width
            CustomText {
                id: calcButtonText
                anchors.centerIn: parent                
                text: qsTr("Calc")
            }            
        }
        CustomButton {
            id: moleculesButton
            anchors.left: calcButton.right
            anchors.top: spectrumPlot.top
            anchors.margins: menuButton.anchors.topMargin
            anchors.leftMargin: menuButtonText.width * 0.5
            implicitHeight: menuButton.height
            implicitWidth: menuButton.width
            CustomText {
                id: molsButtonText
                anchors.centerIn: parent                
                text: qsTr("Mols")
            }
        }
        CustomButton {
            id: exitButton
            anchors.right: spectrumPlot.right
            anchors.top: spectrumPlot.top
            anchors.margins: menuButton.anchors.topMargin
            implicitHeight: menuButton.height
            implicitWidth: menuButton.width
            CustomText {
                id: exitButtonText
                anchors.centerIn: parent
                text: qsTr("Exit")
            }
        }

        Rectangle {
            id: showTotalOptionsRectangle
            anchors.top: exitButton.bottom
            anchors.bottom: totalMolsButton.top
            anchors.right: spectrumPlot.right
            anchors.left: exitButton.left
            anchors.margins: menuButton.height * 0.1
            visible: false

            ColumnLayout {
                id: showTotalOptionsColumnLayout
                anchors.centerIn: parent

                Button {
                    id: showPlainTotalButton
                    anchors.right: showTotalOptionsColumnLayout.right
                    anchors.left: showTotalOptionsColumnLayout.left
                    anchors.margins: 1
                    checkable: true
                    onCheckedChanged: specResultsDataObj.showPlainTotal = checked;
                    checked: specResultsDataObj.showPlainTotal
                    implicitHeight: menuButton.width * 0.95 - 3
                    implicitWidth: menuButton.width * 0.95
                    background: Rectangle {
                        border.color: 'black'
                        border.width: 2
                        color: showPlainTotalButton.checked ? 'black' : 'white'
                        radius: height * 0.5
                    }
                }

                Button {
                    id: showInstrTotalButton
                    anchors.right: showTotalOptionsColumnLayout.right
                    anchors.left: showTotalOptionsColumnLayout.left
                    anchors.margins: 2
                    checkable: true
                    onCheckedChanged: specResultsDataObj.showInstrTotal = checked;
                    checked: specResultsDataObj.showInstrTotal
                    implicitHeight: menuButton.width * 0.95 - 3
                    implicitWidth: menuButton.width * 0.95
                    background: Rectangle {
                        border.color: 'red'
                        border.width: 2
                        color: showInstrTotalButton.checked ? 'red' : 'white'
                        radius: height * 0.5
                    }
                }

                Button {
                    id: showTotalOrRadianceButton
                    anchors.right: showTotalOptionsColumnLayout.right
                    anchors.left: showTotalOptionsColumnLayout.left
                    anchors.margins: 2
                    checkable: true
                    onCheckedChanged: specResultsDataObj.showRadiance = checked
                    checked: specResultsDataObj.showRadiance
                    implicitHeight: menuButton.width * 0.95 - 3
                    implicitWidth: menuButton.width * 0.95
                    CustomText {
                        text: "R"
                        anchors.centerIn: parent
                        color: showTotalOrRadianceButton.checked ? 'white' : 'grey'
                    }
                    background: Rectangle {
                        id: auxRect
                        border.color: 'grey'
                        border.width: 2
                        color: showTotalOrRadianceButton.checked ? 'grey' : 'white'
                        radius: height * 0.5
                    }
                }
            }
        }

        CustomButton {
            id: totalMolsButton
            anchors.bottom: spectrumPlot.bottom
            anchors.right: spectrumPlot.right
            anchors.rightMargin: menuButton.anchors.topMargin
            anchors.bottomMargin: spectrumPlot.yMaxPadding
            checkable: true
            onCheckedChanged: specResultsDataObj.showTotal = checked;
            checked: specResultsDataObj.showTotal
            clip: true
            implicitHeight: menuButton.height
            implicitWidth: menuButton.width - 1
            CustomText {
                id: totalMolsButtonText
                anchors.centerIn: parent
                text: qsTr("Total")
            }
        }
    }
}
